package com.example.what_the_fantasy.data.retrofit

import com.example.what_the_fantasy.data.model.Image
import com.example.what_the_fantasy.data.model.LangAdapter
import com.example.what_the_fantasy.data.model.SrcTypeAdapter
import com.example.what_the_fantasy.data.model.Quote
import com.example.what_the_fantasy.data.model.SrcLanguage
import com.example.what_the_fantasy.data.model.SrcType
import com.example.what_the_fantasy.data.model.User
import com.example.what_the_fantasy.data.services.APIReponceList
import com.google.gson.Gson
import com.google.gson.GsonBuilder
import retrofit2.Response
import retrofit2.Retrofit
import retrofit2.converter.gson.GsonConverterFactory
import retrofit2.http.Body
import retrofit2.http.GET
import retrofit2.http.POST
import retrofit2.http.PUT
import retrofit2.http.Path
import retrofit2.http.Query


interface ApiService {

    @GET("users/username")
    suspend fun getUserByUsername(
        @Query("username") username: String
    ): User

    @GET("users/{id}")
    suspend fun getUserById(
        @Path("id") id: Int
    ): User

    @GET("users/existusername")
    suspend fun checkIfUsernameExists(
        @Query("username") username: String
    ): Boolean

    @GET("users/existemail")
    suspend fun checkIfEmailExists(
        @Query("email") email: String
    ): Boolean


    @PUT("users")
    suspend fun updateUsername(
        @Query("id") id: Int,
        @Body user: User // Envoie un objet `User` avec les nouvelles données
    ): Response<Unit>




    @GET("quote/dailyquote")
    suspend fun getDailyQuote(
        @Query("year") year: Int,
        @Query("month") month: Int,
        @Query("day") day: Int,
        @Query("lang") lang: SrcLanguage
    ): Quote

    @GET("quote/{id}")
    suspend fun getQuoteById(
        @Path("id")id : Int
    ):Quote

    @GET("quote/all")
    suspend fun getAllQuote(
        @Query("index") index: Int,
        @Query("count") count: Int
    ): APIReponceList<Quote>


    @GET("image/all")
    suspend fun getAllImages(
        @Query("index") index: Int,
        @Query("count") count: Int
    ): List<Image>

    @GET("image/{id}")
    suspend fun getImageById(
        @Path("id") id: Int
    ): Image
}


object RetrofitInstance {
    private const val BASE_URL = "https://codefirst.iut.uca.fr/containers/WhatTheFantasy-web-services/api/v1/"

    private val gson: Gson = GsonBuilder()
        .registerTypeAdapter(SrcLanguage::class.java, LangAdapter())
        .registerTypeAdapter(SrcType::class.java, SrcTypeAdapter())
        .create()

    val api: ApiService by lazy {
        Retrofit.Builder()
            .baseUrl(BASE_URL)
            .addConverterFactory(GsonConverterFactory.create(gson))
            .build()
            .create(ApiService::class.java)
    }
}